/*
 * Copyright (C) 2014 Sony Mobile Communications AB.
 * All rights, including trade secret rights, reserved.
 */
package com.sonymobile.ds4;

import android.bluetooth.BluetoothDevice;
import com.sonymobile.ds4.IHidSonyDS4Callback;
import com.sonymobile.ds4.MotionReport;
import com.sonymobile.ds4.TouchReport;


/**
 * IHidSonyDS4Service
 *
 * NOTE:
 * application cannot bind when turn off bluetooth. so app need to check state of bluetooth adapter
 * before execute bindService.
 */
interface IHidSonyDS4Service {

    /**
     * registerCallback
     *
     * regiter inherited class of IHidSonyDS4Callback.
     *
     * @param   callback
     */
    void registerCallback(IHidSonyDS4Callback callback);

    /**
     * unregisterCallback
     *
     * unregiter inherited class of IHidSonyDS4Callback.
     *
     * NOTE:
     * application should execute this method exactly when do unbindService.
     *
     * @param   callback
     */
    void unregisterCallback(IHidSonyDS4Callback callback);

    /**
     * setMotor
     *
     * set DS4 moter mode.
     *
     * @param   device      connected bluetooth device
     * @param   big         value of big motor
     * @param   smoll       value of small motor
     * @return  true if succeed, otherwise false
     */
    boolean setMotor(in BluetoothDevice device, int big, int small);

    /**
     * setLightBar
     *
     * set DS4 light bar color.
     *
     * @param   device              connected bluetooth device
     * @param   red, green, blue    value of colors
     * @return  true if succeed, otherwise false
     */
    boolean setLightBar(in BluetoothDevice device, int red, int green, int blue);

    /**
     * setLightBarBlink
     *
     * set DS4 light bar blink.
     *
     * @param   device              connected bluetooth device
     * @param   onTime, offTime     on/off time of blinking
     * @return  true if succeed, otherwise false
     */
    boolean setLightBarBlink(in BluetoothDevice device, int onTime, int offTime);

    /**
     * resetOrientation
     *
     * Motion Utility API, reset controller's Orientation value.
     *
     * @param   device      connected bluetooth device
     * @return  true if succeed, otherwise false
     */
    boolean resetOrientation(in BluetoothDevice device);

    /**
     * setTiltCollectionFilter
     *
     * Motion Utility API, enable/disable tilt correction filter on controller orientation data.
     *
     * @param   device      connected bluetooth device
     * @param   enable      true to enable sce Motion Utility Collection Filter
     * @return  true if succeed, otherwise false
     */
    boolean setTiltCollectionFilter(in BluetoothDevice device, boolean enable);

    /**
     * setDeadbandFilter
     *
     * Motion Utility API, enable/disable deadband filter of gyro sensor.
     *
     * @param   device      connected bluetooth device
     * @param   enable      true to enable sce Motion Utility Deadband Filter
     * @return  true if succeed, otherwise false
     */
    boolean setDeadbandFilter(in BluetoothDevice device, boolean enable);

    /**
     * getMotionReports
     *
     * get last 64 motion reports.
     *
     * @param   device      connected bluetooth device
     * @return  list of MotionReport class if succeed, otherwise null
     */
    List<MotionReport> getMotionReports(in BluetoothDevice device);

    /**
     * getTouchReports
     *
     * get last 64 touch reports.
     *
     * @param   device      connected bluetooth device
     * @return  list of TouchReport class if succeed, otherwise null
     */
    List<TouchReport> getTouchReports(in BluetoothDevice device);

    /**
     * setMotionReportInterval
     *
     * set interval of onMotionReport occurrence.
     *
     * @param   device      connected bluetooth device
     * @param   time        milliseconds for interval
     * @return  true if succeed, otherwise false
     */
    boolean setMotionReportInterval(in BluetoothDevice device, int time);

    /**
     * isDS4
     *
     * check Bluetooth device is DUALSHOCK4. (unrelated to connection status)
     *
     * @param   device      bluetooth device
     * @return  true is dualshock4, otherwise false
     */
    boolean isDS4(in BluetoothDevice device);

    /**
     * isDS4ByInputDeviceID
     *
     * check intent is from DUALSHOCK4.
     *
     * @param   id          Android key event device ID
     * @return  true is dualshock4, otherwise false
     */
    boolean isDS4ByInputDeviceID(int id);

    /**
     * getDS4No
     *
     * DS4 reference number, it can be used to set light bar color.
     *
     * @param   device      connected bluetooth device
     * @return  0:BLUE, 1:RED, 2:GREEN, 3:PINK, -1:error
     */
    int getDS4No(in BluetoothDevice device);

    /**
     * getDS4NoByInputDeviceId
     *
     * get DS4 reference number by Android key event device ID.
     *
     * @param   id          Android key event device ID
     * @return  DS4 Number (0,1,2,3) if succeed, otherwise -1
     */
    int getDS4NoByInputDeviceId(int id);

    /**
     * getBtDeviceByDS4No
     *
     * get Bluetooth device by DS4 No.
     *
     * @param   ds4no       DS4 Number (0,1,2,3)
     * @return  BluetoothDevice class if succeed, otherwise null
     */
    BluetoothDevice getBtDeviceByDS4No(int ds4no);

    /**
     * getBtDeviceByInputDeviceID
     *
     * get Bluetooth device by Android key event device ID.
     *
     * @param   id          Android key event device ID
     * @return  BluetoothDevice class if succeed, otherwise null
     */
//    BluetoothDevice getBtDeviceByInputDeviceID(int id);

    /**
     * getInputDeviceIdByDS4No
     *
     * get InputDevice ID (Android key event device ID) by DS4 No.
     *
     * @param   ds4no       DS4 Number (0,1,2,3)
     * @return  InputDevice ID if succeed, otherwise -1
     */
    int getInputDeviceIdByDS4No(int ds4no);

    /**
     * getInputDeviceIdByBtDevice
     * get InputDevice ID (Android key event device ID) by Bluetooth device
     *
     * @param   device      connected bluetooth device
     * @return  InputDevice ID if succeed, etherwise -1
     */
//    int getInputDeviceIdByBtDevice(in BluetoothDevice device);
}
